/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2013 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xml.type.AnyType;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.extension.IMigrationProvider;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;

/**
 * {@link IMigrationProvider} that ensures that all {@link IAnnotatedSpecification}s that have been
 * registered for model element types in the current model are actually instantiated.
 * 
 * This is required for new {@link IAnnotatedSpecification}s that have been registered after the
 * creation of the model.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: DB938B8077EEAFB9455F8A83BF92CD86
 */
public class AnnotationInstantiationMigrationProvider implements IMigrationProvider {

	/**
	 * Set of models already processed during this run. The implementation of a predicate function
	 * if model migration is needed (for use in {@link #needMigration(ITopLevelElement, Map)} would
	 * be as complex as the migration itself.
	 */
	private Set<ITopLevelElement> processedModels = new HashSet<ITopLevelElement>();

	/** {@inheritDoc} */
	@Override
	public boolean needMigration(ITopLevelElement modelElement,
			Map<EObject, AnyType> unknownFeatures) {

		return modelElement != null && modelElement.getRootModelElement() != null &&
				!processedModels.contains(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public void migrate(ITopLevelElement modelElement, Map<EObject, AnyType> unknownFeatures) {
		processedModels.add(modelElement);

		for(Iterator<EObject> i = modelElement.getRootModelElement().eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IModelElement) {

				// IAnnotationValueService.INSTANCE.instantiateAnnotations((IModelElement)eo);
			}
		}
	}
}
