/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Base class for {@link EditingSupport}s used in the {@link TableViewer}-based views.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED
 */
public abstract class AnnotationEditingSupportBase extends EditingSupport {

	/** Value used to allow user to unset annotation value. */
	protected final static String UNSET_RESERVED_VALUE = "";

	/** Specification class of this column */
	protected Class<? extends IAnnotatedSpecification> specClass;

	/** Instance key to identify the corresponding instance of the annotation */
	protected String instanceKey;

	/** Constructor. */
	public AnnotationEditingSupportBase(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> specClass, String instanceKey) {
		super(viewer);
		this.specClass = specClass;
		this.instanceKey = instanceKey;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry annotationEntry = (AnnotationEntry)element;
			return annotationEntry.canEdit(specClass, instanceKey);
		}

		return true;
	}

	/** Actually gets the value from the {@link IAnnotatedSpecification}. */
	protected Object doGetValue(AnnotationEntry annotationEntry) {
		return annotationEntry.getSpecificationValue(specClass, instanceKey);
	}

	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		if(element instanceof AnnotationEntry) {
			return doGetValue((AnnotationEntry)element);
		}
		return null;
	}

	/** Actually sets a {@link String} value into a {@link IAnnotatedSpecification}. */
	protected void doSetValue(AnnotationEntry annotationEntry, String value) throws Exception {
		annotationEntry.setSpecificationValue(value, specClass, instanceKey);
	}

	/** Actually sets a value into a {@link IAnnotatedSpecification}. */
	protected void doSetValue(AnnotationEntry annotationEntry, Object value) throws Exception {
		annotationEntry.setSpecificationValue(value, specClass, instanceKey);
	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(Object element, final Object value) {
		if(element instanceof AnnotationEntry) {
			final AnnotationEntry annotationEntry = (AnnotationEntry)element;

			ITopLevelElement modelContext =
					IPersistencyService.INSTANCE.getTopLevelElementFor(annotationEntry
							.getModelElement());
			modelContext.runAsCommand(new Runnable() {

				@Override
				public void run() {

					try {
						doSetValue(annotationEntry, value);
					} catch(IllegalArgumentException e) {
						MessageDialog
								.openError(
										new Shell(),
										"ERROR",
										"The value you entered does not have the desired type.\nDetailed message:\n" +
												e.getMessage());
					} catch(Exception e) {
						MessageDialog.openError(new Shell(), "ERROR", e.getMessage());
					}
				}
			});
			this.getViewer().refresh(true);

		}
	}
}
