/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * {@link EditingSupport} that allows to edit the comment of {@link INamedCommentedElement}s.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED
 */
public class ElementCommentEditingSupport extends ElementEditingSupportBase {

	/** Constructs a new {@link ElementCommentEditingSupport}. */
	public ElementCommentEditingSupport(ColumnViewer viewer) {
		super(viewer);
	}

	/** {@inheritDoc} */
	@Override
	protected Object doGetValue(AnnotationEntry annotationEntry) {
		IModelElement modelElement = annotationEntry.getModelElement();

		if(modelElement instanceof INamedCommentedElement) {
			return ((INamedCommentedElement)modelElement).getComment();
		}

		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void doSetValue(AnnotationEntry annotationEntry, String value) throws Exception {
		IModelElement modelElement = annotationEntry.getModelElement();

		if(modelElement instanceof INamedCommentedElement) {
			((INamedCommentedElement)modelElement).setComment(value);
		}
	}
}
