/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.ComponentEditPolicy;
import org.eclipse.gef.requests.GroupRequest;
import org.fortiss.tooling.base.ui.editpart.command.RemoveCommand;
import org.fortiss.tooling.kernel.service.IElementCompositorService;

/**
 * An edit policy for removing model elements. Delete commands are redirected to
 * {@link IElementCompositorService} to determine the model change.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 998816E233579A017ECF221071C5AE43
 */
public final class RemoveEditPolicy extends ComponentEditPolicy {

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Command createDeleteCommand(GroupRequest request) {
		CompoundCommand compound = new CompoundCommand();
		for(EditPart ep : (List<EditPart>)request.getEditParts()) {
			EObject model = (EObject)ep.getModel();
			boolean canDelete = IElementCompositorService.INSTANCE.canDecompose(model);
			if(canDelete) {
				compound.add(new RemoveCommand(model));
			} else {
				return UnexecutableCommand.INSTANCE;
			}
		}
		return compound;
	}
}
