/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.annotation;

import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.utils.KernelModelElementUtils;

/**
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 89623945B1C7BB21FA0E610249272368
 */
public interface IAnnotationValueService {

	/** Singleton instance */
	public static final IAnnotationValueService INSTANCE = new AnnotationValueService();

	/**
	 * <p>
	 * Returns the {@link AnnotationEntry} for the given {@link IModelElement}. In case any of the
	 * {@link IAnnotatedSpecification}s registered for the corresponding {@link IModelElement}
	 * specialization does not exist for the given {@code modelElement} yet, it instantiated.
	 * </p>
	 * 
	 * <p>
	 * This method uses
	 * {@link KernelModelElementUtils#runAsCommand(org.eclipse.emf.ecore.EObject, Runnable)} and
	 * hence is safe to be called from the GUI context.
	 * </p>
	 */
	public AnnotationEntry getAnnotationEntry(IModelElement element);

	/**
	 * <p>
	 * If required, instantiate annotation for the given model element.
	 * </p>
	 * 
	 * <p>
	 * <b>CAVEAT: </b>This method must be called from a context that allows the modification of the
	 * model.
	 * </p>
	 */
	public void instantiateAnnotations(IModelElement element);

}
