/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.ILibraryElementHandler;
import org.fortiss.tooling.kernel.extension.base.PrototypeProviderBase;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.PrototypeCategory;
import org.fortiss.tooling.kernel.model.ILibraryElement;
import org.fortiss.tooling.kernel.model.ILibraryPackage;
import org.fortiss.tooling.kernel.service.ILibraryService;

/**
 * The base class for prototype providers for library elements.
 * 
 * @author ratiu
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating GREEN Hash: 95B48E4539DB312738E530B05CE9483A
 */
public class LibraryPrototypeProvider extends PrototypeProviderBase {

	/**
	 * Refreshes the list of prototypes to synchronize them with the current state
	 * of the library.
	 */
	public void refreshPrototypes() {
		prototypes.clear();
		categories.clear();
		registerPrototypes();
	}

	/** {@inheritDoc} */
	@Override
	protected void registerPrototypes() {
		PrototypeCategory libraryCategory = registerPrototypeCategory("Library");
		Map<String, PrototypeCategory> registeredCategoriesMap =
				new HashMap<String, PrototypeCategory>();

		for(ILibraryElement libElem : ILibraryService.INSTANCE
				.getLibraryElementsFromAllWorkspaceLibraries()) {
			List<ILibraryElementHandler<EObject>> handlers =
					((LibraryService)ILibraryService.INSTANCE).getRegisteredHandlers(libElem
							.getWrappedElement().getClass());

			ILibraryElementHandler<EObject> firstHandler = handlers.get(0);

			Prototype prot = firstHandler.createLibraryElementReferencePrototype(libElem);

			prototypes.add(prot);

			String categoryName = ((ILibraryPackage)libElem.eContainer()).getName();
			PrototypeCategory current = registeredCategoriesMap.get(categoryName);

			if(current == null) {
				current = registerPrototypeCategory(libraryCategory, categoryName);
				registeredCategoriesMap.put(categoryName, current);
			}

			current.add(prot);
		}
	}
}
