/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.jface;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Control;

/**
 * Utility class for JFace columns.
 * 
 * @author deissenb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: 391516A80A92354403CB0D7B0615FDF4
 */
public class ColumnUtils {

	/**
	 * This method initialize a table viewer with the provided columns. It sets
	 * the header visible and adds each column with title, width and column
	 * label provider.
	 */
	public static void initTableColumns(TableViewer tableViewer,
			IColumn... columns) {
		tableViewer.getTable().setHeaderVisible(true);
		for (IColumn column : columns) {
			TableViewerColumn viewerColumn = new TableViewerColumn(tableViewer,
					column.getAlignment());
			viewerColumn.getColumn().setText(column.getTitle());
			viewerColumn.getColumn().setWidth(column.getWidth());
			viewerColumn.setLabelProvider(column.getLabelProvider());
		}
	}

	/**
	 * This methods sets the size hints for a {@link GridLayout}. Concretely, it
	 * sets {@link GridData#heightHint} to the specified value and
	 * {@link GridData#widthHint} to the total width of the provided columns.
	 * <p>
	 * <b>Attention:</b> A layout data object previously associated with the
	 * control is overwritten without warning.
	 * 
	 */
	public static void setSizeHint(Control control, int height,
			IColumn... columns) {
		GridDataFactory.fillDefaults().hint(calcTotalWidth(columns), height)
				.applyTo(control);
	}

	/** Calculates the total width of the provided columns. */
	public static int calcTotalWidth(IColumn... columns) {
		int totalWidth = 0;
		for (IColumn column : columns) {
			totalWidth += column.getWidth();
		}
		return totalWidth;
	}
}